//   default effect.
//
//  This is where the shaders are (vertex shader and pixel shader)
//  Edit this file so we use the texture when rendering.
//
//
//  Programming Tip:
//  When using the function D3DXCreateEffectFromFile() to load
//  an effect, you are compiling the shaders on the fly and you
//  dont seem to get as good error messages, or there not in a convenient spot.
//  A useful thing to do is to add shader compilation to the build/compile process.
//  to do this, add your .fx file to your visual studio project.  
//  Since its not a .cpp file, msdev doesn't recognize how to compile it.
//  Right-click on the file and select "settings".
//  Under "Custom Build" add the following commands:
//
//       set FXC="C:\DXSDK\Bin\DXUtils\FXC.EXE"
//       echo compiling vertex_shader
//       set EFFECT=$(InputName)
//       set command=%FXC% %EFFECT%.fx /T vs_2_0 /E vertex_shader /Fc DEBUG\%EFFECT%_vs.txt
//       echo %command%
//       %command%
//       echo Compiling pixel_shader
//       set command=%FXC% %EFFECT%.fx /T ps_2_0 /E pixel_shader /Fc DEBUG\%EFFECT%_ps.txt
//       echo %command%
//       %command%
//
//  I did it this way so it doesn't depend on the name of the effect file.
//  but you will have to change the entry points within this if you didn't call
//  your effects file's functions "vertex_shader" and "pixel_shader".
// 
//  Still under "Custom Build" add
//       DEBUG\$(InputName)_vs.txt
//       DEBUG\$(InputName)_ps.txt
//  to the outputs.
//  Now after you edit your effect .fx file (shaders), you will test compile them
//  the next time you build your application.  Any warning or errors will show
//  up in the build output subwindow, and you can simply double click on  
//  them (or hit function key F4) to go directly to your error.
//










float3 lightposn={0.4,0.4,0.8};  // in world space
float  lightradius;
float3 lightcolor = {0.6,0.6,0.6};
float3 ambient={0.5,0.5,0.5};
float  alpha=0.3;
float  diffusecoef = 1.0;

float3 emissive={0.0,0.0,0.0};
float  gloss = 0.5;   // affects how much specular is reflected.

float4x4 WorldViewProj ;
float4x4 World ;
float4x4 WorldView ;
float4x4 ViewProj ;
float4x4 ViewInv;
float4x4 WorldInv;
float4x4 WorldViewInv;
static const int MAX_MATRICES = 26;
float4x4    WorldMatrixArray[MAX_MATRICES] : WORLDMATRIXARRAY;
int useskin=0;

texture diffusemap : DiffuseMap;

sampler samp = sampler_state
{
	texture = <diffusemap>;
	AddressU = WRAP;
	AddressV = WRAP;
	MIPFILTER = LINEAR;
	MINFILTER = LINEAR;
	MAGFILTER = LINEAR;
};

struct VS_INPUT
{
	float4 position : POSITION;
	float4 normal   : NORMAL;
	float4 texcoord : TEXCOORD;
    float3 tangent	: TANGENT;
    float3 binormal	: BINORMAL;
};

struct VertexS 
{
    float4 position : POSITION;
    float4 weights  : BLENDWEIGHT;
    int4   bones    : BLENDINDICES;
    float4 color	: COLOR0;
    float3 normal	: NORMAL;
    float2 texcoord	: TEXCOORD0;
    float3 tangent	: TANGENT0;
    float3 binormal	: BINORMAL0;
};


struct VtoP
{
	float4 position : POSITION;
	float4 tex      : TEXCOORD;
	float4 color    : COLOR;
	float3 eyepos   : TEXCOORD1;
	float3 lightpos : TEXCOORD2;
	float3 normal   : TEXCOORD3;
};
struct VtoPsimple
{
	float4 position : POSITION;
	float4 tex      : TEXCOORD;
	float4 color    : COLOR;
};

VtoP vertex_shader(const VS_INPUT IN)
{
	VtoP OUT;
	float4 p = mul(IN.position, WorldViewProj);
	OUT.position = p;
	float3 pw = mul(IN.position, World).xyz;
	float3 lightdir = normalize(lightposn - pw.xyz);
	float3 viewdir  = normalize(ViewInv[3].xyz -pw.xyz);
	float3 eye_local = WorldViewInv[3].xyz;
	OUT.eyepos = eye_local - IN.position.xyz;
	float4 lightposn4;
	lightposn4.xyz = lightposn;
	lightposn4.w = 1.0;
	float3 light_local = mul(lightposn4,WorldInv).xyz;
	OUT.lightpos = light_local - IN.position.xyz;
	OUT.normal = IN.normal.xyz;
	float3 n = mul(IN.normal.xyz,World);
	float dp = dot(n,lightdir);
	dp = clamp(dp,0,1);
	OUT.color.xyz =  emissive + ambient + lightcolor*dp;
	OUT.color.w = 1.0;
	OUT.tex = IN.texcoord;

	return OUT;
}

VtoP vertex_shader_skin(VertexS IN)
{
	int4 bones = IN.bones; // unlike skinnedmesh.fx from d3d sample, they used float instead of int.
	float4 pos = float4(0,0,0,0);
	VtoP OUT = (VtoP)0;

	// vertex position in object space
	//float4 Po=float4(IN.position.xyz,1.0);
	pos += mul(IN.position,WorldMatrixArray[bones.x]) * IN.weights.x;
	pos += mul(IN.position,WorldMatrixArray[bones.y]) * IN.weights.y;
	pos += mul(IN.position,WorldMatrixArray[bones.z]) * IN.weights.z;
	pos += mul(IN.position,WorldMatrixArray[bones.w]) * IN.weights.w;

	float3 normal=float3(0,0,0);
	normal += mul(IN.normal,WorldMatrixArray[bones.x]) * IN.weights.x;
	normal += mul(IN.normal,WorldMatrixArray[bones.y]) * IN.weights.y;
	normal += mul(IN.normal,WorldMatrixArray[bones.z]) * IN.weights.z;
	normal += mul(IN.normal,WorldMatrixArray[bones.w]) * IN.weights.w;
	float3 tangent=float3(0,0,0);
	tangent += mul(IN.tangent,WorldMatrixArray[bones.x]) * IN.weights.x;
	tangent += mul(IN.tangent,WorldMatrixArray[bones.y]) * IN.weights.y;
	tangent += mul(IN.tangent,WorldMatrixArray[bones.z]) * IN.weights.z;
	tangent += mul(IN.tangent,WorldMatrixArray[bones.w]) * IN.weights.w;
	float3 binormal=float3(0,0,0);
	binormal += mul(IN.binormal,WorldMatrixArray[bones.x]) * IN.weights.x;
	binormal += mul(IN.binormal,WorldMatrixArray[bones.y]) * IN.weights.y;
	binormal += mul(IN.binormal,WorldMatrixArray[bones.z]) * IN.weights.z;
	binormal += mul(IN.binormal,WorldMatrixArray[bones.w]) * IN.weights.w;
	// compute modelview rotation only part
	//float3x3 modelviewrot;
	//modelviewrot[0]=WorldView[0].xyz;
	//modelviewrot[1]=WorldView[1].xyz;
	//modelviewrot[2]=WorldView[2].xyz;

	// vertex position in clip space
	OUT.position = mul(pos,ViewProj);

	float3 pw = mul(IN.position, World).xyz;
	float3 lightdir = normalize(lightposn - pos.xyz);
	float3 viewdir  = normalize(ViewInv[3].xyz -pos.xyz);
	float3 eye_world = ViewInv[3].xyz;
	OUT.eyepos = eye_world - pos.xyz;  // surface to eye vector in world space (rotation)
	float3 light_local = 
	OUT.lightpos  = lightposn - pos.xyz; // surface to light vector in world space 
	OUT.normal    = normal;
	//OUT.binormal  = binormal;
	//OUT.tangent   = tangent;
	OUT.color     = IN.color ; 
	OUT.tex.x    = IN.texcoord.x;
	OUT.tex.y    = 1.0-IN.texcoord.y;  // Prefer origin in bottom left - consistent with 3dsmax and with OpenGL
	OUT.tex.z     = 0;
	OUT.tex.w     = 1;

	return OUT;
}
VertexShader vsArray[2] = { compile vs_2_0 vertex_shader(), 
                            compile vs_2_0 vertex_shader_skin(),
};


struct PS_OUTPUT
{
	float4 color : COLOR0;
};

VtoPsimple vertex_shader_simple(const VS_INPUT IN)
{
	VtoPsimple OUT;
	float4 p = mul(IN.position, WorldViewProj);
	OUT.position = p;
	float3 pw = mul(IN.position, World).xyz;
	float3 lightdir = normalize(lightposn - pw.xyz);
	float3 n = mul(IN.normal.xyz,World);
	float dp = dot(n,lightdir);
	dp = clamp(dp,0,1);
	OUT.color.xyz =  emissive + ambient + lightcolor*dp;
	OUT.color.w = 1.0;
	OUT.tex = IN.texcoord;

	return OUT;
}
float4 pixel_shader_simple(const VtoPsimple IN):COLOR
{
	return IN.color * tex2D(samp, IN.tex);
}


float3 specularcolor = {0.25,0.25,0.25};

float4 pixel_shader(const VtoP IN) :COLOR
{
	float4 color;
	float3 e = normalize(IN.eyepos);
	float  falloff = saturate(1-length(IN.lightpos)/lightradius);
	float3 l = normalize(IN.lightpos);
	float3 n = normalize(IN.normal);
	float3 h = normalize(e+l);
	float spec = pow(saturate(dot(n,h)),64);  // specular intensity
	float d = saturate(dot(n,l));
	color.xyz = saturate(emissive + ambient + diffusecoef * lightcolor * d * falloff)  * tex2D(samp, IN.tex) + lightcolor * gloss * spec * falloff;
	color.w = 1.0;
	return color;
}


float4 pixel_shader_light(const VtoP IN) :COLOR
{
	float4 color;
	float  falloff = saturate(1-length(IN.lightpos)/lightradius);
	float3 e = normalize(IN.eyepos);
	float3 l = normalize(IN.lightpos);
	float3 n = normalize(IN.normal);
	float3 h = normalize(e+l);
	float spec = pow(saturate(dot(n,h)),64);  // specular intensity
	float d = saturate(dot(n,l));
	color.xyz = ((diffusecoef * lightcolor * d)  * tex2D(samp, IN.tex) + lightcolor * gloss * spec) * falloff;
	color.w = 1.0;
	return color;
}



float4 pixel_shader_unlit(const VtoP IN) :COLOR
{
	float4 color;
	color.xyz = saturate(emissive + ambient)  * tex2D(samp, IN.tex);
	color.w = 1.0;
	return color;
}

technique t0
{
	pass p0
	{
		DEPTHBIAS = 0;
		TWOSIDEDSTENCILMODE = FALSE;
		CullMode = CW;
		STENCILENABLE=    FALSE  ;
		ALPHABLENDENABLE= FALSE ;
		SHADEMODE = GOURAUD ;
		ZWRITEENABLE =     TRUE ;
		ZENABLE = TRUE;
		ZFUNC = LESSEQUAL ;
		PixelShader  = compile ps_2_0 pixel_shader();
		VertexShader = vsArray[useskin]; //compile vs_2_0 vertex_shader();
	}
}
technique mirrored
{
	pass p0
	{
		DEPTHBIAS = 0;
		TWOSIDEDSTENCILMODE = FALSE;
		CullMode = CCW;
		STENCILENABLE=    FALSE  ;
		ALPHABLENDENABLE= FALSE ;
		SHADEMODE = GOURAUD ;
		ZWRITEENABLE =     TRUE ;
		ZENABLE = TRUE;
		ZFUNC = LESSEQUAL ;
		PixelShader  = compile ps_2_0 pixel_shader();
		VertexShader = compile vs_2_0 vertex_shader();
	}
}

technique unlit 
{
	pass p0
	{
		DEPTHBIAS = 0;
		TWOSIDEDSTENCILMODE = FALSE;
		ZENABLE = TRUE;
		ZFUNC = LESSEQUAL ;
		ZWRITEENABLE =     TRUE ;
		CullMode = CW;
		STENCILENABLE=    FALSE  ;
		ALPHABLENDENABLE= FALSE ;
		SHADEMODE = GOURAUD ;
		PixelShader  = compile ps_2_0 pixel_shader_unlit();
		VertexShader = vsArray[useskin];//compile vs_2_0 vertex_shader();
	}
}

technique lightpass
{
	pass p0
	{
		DepthBias = -0.00001f;  // nvidia 6200go and ati x800 doesn't need this , but other ati cards may need it if clipped polys aren't coplanar 
		STENCILENABLE=    TRUE  ;
		TWOSIDEDSTENCILMODE = FALSE;
		CCW_STENCILPASS=KEEP;
		STENCILPASS=     KEEP;
		STENCILFAIL=     KEEP;
		STENCILZFAIL=     KEEP;
		ZWRITEENABLE=     FALSE ;
		ZENABLE = TRUE;
		ZFUNC = LESSEQUAL ;
		STENCILREF=  0x01 ;
		STENCILFUNC= GREATER; // since unlike shadowpoly we want to draw where shadow isn't  // LESSEQUAL ;
		FOGENABLE=        FALSE ;
		ALPHABLENDENABLE= TRUE  ;
		SHADEMODE = GOURAUD ;
		SRCBLEND=  ONE ;
		DESTBLEND= ONE ;
		CullMode = CW;
		FILLMODE=SOLID;
		PixelShader  = compile ps_2_0 pixel_shader_light();
		VertexShader = vsArray[useskin];// compile vs_2_0 vertex_shader();
	}
}

struct VtoP_deferred  // all elements are in world space
{
	float4 position : POSITION;  
	float4 tex      : TEXCOORD;
	//float4 color  : COLOR;
	float4 worldpos : TexCoord1;
	float3 tangent  : TEXCOORD2;
	float3 binormal : TEXCOORD3;
	float3 normal   : TEXCOORD4;
	float4 position2: TEXCOORD5;  // identical to position, but accessible from pixel shader
};

struct psOut_deferred
{
	float4 diffuse  : Color0;
	float4 normals  : Color1;  // packed into range 0..1 on each xyz
	float4 position : Color2;  // actually we just use one F32 channel for depth info, but its used to derive position later on
};

VtoP_deferred vertex_shader_deferred_setup(const VS_INPUT IN)
{
	VtoP_deferred OUT;
	OUT.position = mul(IN.position, WorldViewProj);
	OUT.position2= mul(IN.position, WorldView);
	OUT.worldpos = mul(IN.position, World);
	OUT.tangent  = mul(float4(IN.tangent.xyz ,0.0f),World).xyz;
	OUT.binormal = mul(float4(IN.binormal.xyz,0.0f),World).xyz;
	OUT.normal   = mul(float4(IN.normal.xyz  ,0.0f),World).xyz;
	OUT.tex      = IN.texcoord;
	return OUT;
}
psOut_deferred pixel_shader_deferred_setup(VtoP_deferred In)
{
	psOut_deferred Out;
	float4 d = tex2D(samp, In.tex);
	//Out.color   = float4(emissive + ambient,1.0f) * d;
	Out.diffuse = d;
	float3 normal = normalize(In.normal); //  mul(2.0f * tex2D(sNormalMap, In.tex) - 1.0f, float3x3(In.tangent, In.binormal, In.normal));
	Out.normals.xyz = float3(0.5,0.5,0.5)+normal*0.5;
	Out.normals.w = 1.0f;
 	//Out.position.xyzw = In.position2.z / In.position2.w;    // depth
	Out.position.xyzw = -length(In.worldpos-ViewInv[3].xyz);  // ok this is actually distance, not quite the same as .z type of depth
	return Out;
}

technique deferred_setup
{
	pass p0
	{
		DEPTHBIAS = 0;
		TWOSIDEDSTENCILMODE = FALSE;
		CullMode = CW;
		STENCILENABLE=    FALSE  ;
		ALPHABLENDENABLE= FALSE ;
		SHADEMODE = GOURAUD ;
		ZWRITEENABLE =     TRUE ;
		ZENABLE = TRUE;
		ZFUNC = LESSEQUAL ;
		PixelShader  = compile ps_2_0 pixel_shader_deferred_setup();
		VertexShader = compile vs_2_0 vertex_shader_deferred_setup();
	}
}

// make a lame fallback technique here for 1.1

technique simple
{
	pass p0
	{
		DEPTHBIAS = 0;
		TWOSIDEDSTENCILMODE = FALSE;
		ZENABLE = TRUE;
		CullMode = CW;
		STENCILENABLE=    FALSE  ;
		ALPHABLENDENABLE= FALSE ;
		SHADEMODE = GOURAUD ;
		ZWRITEENABLE =     TRUE ;
		VertexShader = compile vs_2_0 vertex_shader_simple();
		PixelShader  = compile ps_2_0 pixel_shader_simple();
	}
}


VtoP vertex_shader_alpha(const VS_INPUT IN)
{
	VtoP OUT = (VtoP)0;
	float4 p = mul(IN.position, WorldViewProj);
	OUT.position = p;
	OUT.color.xyz =  1.0;
	OUT.color.w = alpha;
	OUT.tex = IN.texcoord;
	return OUT;
}

PS_OUTPUT pixel_shader_alpha(const VtoP IN)
{
	PS_OUTPUT OUT;
	OUT.color = IN.color * tex2D(samp, IN.tex);
	return OUT;
}

technique alpha
{
	pass p0
	{
		DEPTHBIAS = 0;
		TWOSIDEDSTENCILMODE = FALSE;
		ZENABLE = TRUE;
		ZFUNC   = LESSEQUAL ;
		CullMode = CW;
		STENCILENABLE=    FALSE  ;
		ALPHABLENDENABLE= TRUE ;
		SHADEMODE = GOURAUD ;
		ZWRITEENABLE =     FALSE ;
        SRCBLEND  = SRCALPHA ;
        DESTBLEND = INVSRCALPHA ;
		PixelShader  = compile ps_2_0 pixel_shader_alpha();
		VertexShader = compile vs_2_0 vertex_shader_alpha();
	}
	pass p1
	{
		DEPTHBIAS = 0;
		TWOSIDEDSTENCILMODE = FALSE;
		ZENABLE = TRUE;
		ZFUNC   = LESSEQUAL ;
		CullMode = CCW;
		STENCILENABLE=    FALSE  ;
		ALPHABLENDENABLE= TRUE ;
		SHADEMODE = GOURAUD ;
		ZWRITEENABLE =     FALSE ;
        SRCBLEND  = SRCALPHA ;
        DESTBLEND = INVSRCALPHA ;
		PixelShader  = compile ps_2_0 pixel_shader_alpha();
		VertexShader = compile vs_2_0 vertex_shader_alpha();
	}
}

