/*********************************************************************NVMH3****
File:  $Id: //sw/devtools/ShaderLibrary/1.0/HLSL/relief_mapping.fx#3 $

Copyright NVIDIA Corporation 2007
TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
*AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

% This material shows and compares results from four popular and
% advanced schemes for emulating displaement mapping.  They are:
% Relief Mapping, Parallax Mapping, Normal Mapping, and Relief
% Mapping with Shadows.  Original File by Fabio Policarpo.

keywords: material bumpmap
date: 070305


******************************************************************************/

//
// Notice the att term used in the following line of code:
//     float att=saturate(dot(l,IN.normal.xyz));
//
//  This additional coeficient in the lighting equations isn't commonly used.
//  It does greatly reduce specular reflecting at low angles.  Imagine a light at the other end of a long hallway.
//  In fact, according to snell's law and Fresnel's equations, you would get more reflection in this case.
//  More importantly, for deferred shading/lighing, including this term means that 
//  you also have to store the original geometry (unpreturbed) normal.  
//  Fabio's other examples dont use this att term.  
//  Actually they have an att term too but its used for typical attenuation.
//  The benefit from having this term is that it does help with self occlusion.  


float alpha;
int useskin=0;


float TileCount <
	string UIName = "Tile Repeat";
	string UIWidget = "slider";
	float UIMin = 1.0;
	float UIStep = 1.0;
	float UIMax = 32.0;
> = 1;

float Depth <
	string UIName = "Depth";
	string UIWidget = "slider";
	float UIMin = 0.0f;
	float UIStep = 0.001f;
	float UIMax = 0.25f;
> = 0.1;

float3 ambient <
	string UIName = "Ambient";
	string UIWidget = "color";
> = {0.2,0.2,0.2};


float3 lightcolor = {0.5,0.5,0.5} ;
float  gloss = 0.5f;


float PhongExp <
    string UIName = "Phong Exponent";
	string UIWidget = "slider";
	float UIMin = 8.0f;
	float UIStep = 8;
	float UIMax = 256.0f;
> = 64.0;

float3 lightposn : POSITION <
	string UIName="Light Position";
    string Object = "PointLight";
    string Space = "World";	
> = { -150.0, 200.0, -125.0 };

float lightradius;
float lightradiusinv;
/************* UNTWEAKABE VALUES *******/

float4x4 World;
float4x4 WorldViewProj : WorldViewProjection <string UIWidget="none";>;
float4x4 WorldView : WorldView <string UIWidget="none";>;
float4x4 ViewProj : ViewProjection <string UIWidget="none";>;
float4x4 View : View <string UIWidget="none";>;
float4x4 ViewInv;
static const int MAX_MATRICES = 26;
float4x4    WorldMatrixArray[MAX_MATRICES] : WORLDMATRIXARRAY;


/*********** TEXTURES ***************/

texture diffusemap : DIFFUSE <
	string UIName = "Color Texture";
    string ResourceName = "rockwallrm.jpg";
    string ResourceType = "2D";
>;

texture normalmap : NORMAL <
	string UIName = "Normal-Map Texture";
    string ResourceName = "rockwallrm_b.tga";
    string ResourceType = "2D";
>;

sampler2D ColorSamp = sampler_state {
	Texture = <diffusemap>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = WRAP;
	AddressV = WRAP;
};

sampler2D ReliefSamp = sampler_state {
	Texture = <normalmap>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = WRAP;
	AddressV = WRAP;
};

/********** CONNECTOR STRUCTURES *****************/

struct VS_INPUT 
{
    float4 position : POSITION;
    // float4 color	: COLOR0;
    float3 normal	: NORMAL;
    float2 texcoord	: TEXCOORD0;
    float3 tangent	: TANGENT0;
    float3 binormal	: BINORMAL0;
};

struct VertexS 
{
    float4 position : POSITION;
    float4 weights  : BLENDWEIGHT;
    int4   bones    : BLENDINDICES;
    // float4 color	: COLOR0;
    float3 normal	: NORMAL;
    float2 texcoord	: TEXCOORD0;
    float3 tangent	: TANGENT0;
    float3 binormal	: BINORMAL0;
};


struct VertexOutput
{
    float4 hpos		: POSITION;
	float4 color	: COLOR0;
    float2 UV		: TEXCOORD0;
    float3 vpos		: TEXCOORD1;
    float3 tangent	: TEXCOORD2;
    float3 binormal	: TEXCOORD3;
    float3 normal	: TEXCOORD4;
	float4 lightpos	: TEXCOORD5;
};


struct VertexOutputTest
{
    float4 hpos		: POSITION;
};
/*** SHADER FUNCTIONS **********************************************/

VertexOutput vertex_shader_test(VS_INPUT IN)
{
	VertexOutput OUT = (VertexOutput)0;

	// vertex position in object space
	float4 Po=float4(IN.position.xyz,1.0);

	// compute modelview rotation only part
	float3x3 modelviewrot;
	modelviewrot[0]=WorldView[0].xyz;
	modelviewrot[1]=WorldView[1].xyz;
	modelviewrot[2]=WorldView[2].xyz;

	// vertex position in clip space
	OUT.hpos=mul(Po,WorldViewProj);
	return OUT;
}
float4 pixel_shader_test(VertexOutput IN):COLOR
{
	float4 finalcolor;
	finalcolor.xyz = 0.5;
	finalcolor.w = 1.0;
	return finalcolor;
}


VertexOutput vertex_shader_skin(VertexS IN)
{
	int4 bones = IN.bones; // unlike skinnedmesh.fx from d3d sample, they used float instead of int.
	float4 pos = float4(0,0,0,0);
	VertexOutput OUT = (VertexOutput)0;

	// vertex position in object space
	//float4 Po=float4(IN.position.xyz,1.0);
	pos += mul(IN.position,WorldMatrixArray[bones.x]) * IN.weights.x;
	pos += mul(IN.position,WorldMatrixArray[bones.y]) * IN.weights.y;
	pos += mul(IN.position,WorldMatrixArray[bones.z]) * IN.weights.z;
	pos += mul(IN.position,WorldMatrixArray[bones.w]) * IN.weights.w;

	float3 normal=float3(0,0,0);
	normal += mul(IN.normal,WorldMatrixArray[bones.x]) * IN.weights.x;
	normal += mul(IN.normal,WorldMatrixArray[bones.y]) * IN.weights.y;
	normal += mul(IN.normal,WorldMatrixArray[bones.z]) * IN.weights.z;
	normal += mul(IN.normal,WorldMatrixArray[bones.w]) * IN.weights.w;
	float3 tangent=float3(0,0,0);
	tangent += mul(IN.tangent,WorldMatrixArray[bones.x]) * IN.weights.x;
	tangent += mul(IN.tangent,WorldMatrixArray[bones.y]) * IN.weights.y;
	tangent += mul(IN.tangent,WorldMatrixArray[bones.z]) * IN.weights.z;
	tangent += mul(IN.tangent,WorldMatrixArray[bones.w]) * IN.weights.w;
	float3 binormal=float3(0,0,0);
	binormal += mul(IN.binormal,WorldMatrixArray[bones.x]) * IN.weights.x;
	binormal += mul(IN.binormal,WorldMatrixArray[bones.y]) * IN.weights.y;
	binormal += mul(IN.binormal,WorldMatrixArray[bones.z]) * IN.weights.z;
	binormal += mul(IN.binormal,WorldMatrixArray[bones.w]) * IN.weights.w;
	// compute modelview rotation only part
	//float3x3 modelviewrot;
	//modelviewrot[0]=WorldView[0].xyz;
	//modelviewrot[1]=WorldView[1].xyz;
	//modelviewrot[2]=WorldView[2].xyz;

	// vertex position in clip space
	OUT.hpos = mul(pos,ViewProj);

	// vertex position in view space (with model transformations)
	OUT.vpos = mul(pos,View).xyz;

	// light position in view space
	float4 lp=float4(lightposn.xyz,1);
	OUT.lightpos=mul(lp,View);

	// tangent space vectors in view space (with model transformations)
	OUT.tangent=mul(tangent,View);
	OUT.binormal=mul(binormal,View);
	OUT.normal=mul(normal,View);
	
	// copy color and texture coordinates
	OUT.color.xyzw = 1.0 ; // IN.color;
	OUT.UV.x = TileCount * IN.texcoord.x;
	OUT.UV.y = 1.0-TileCount * IN.texcoord.y;

	return OUT;
}

VertexOutput vertex_shader(VS_INPUT IN)
{
	VertexOutput OUT = (VertexOutput)0;

	// vertex position in object space
	float4 Po=float4(IN.position.xyz,1.0);

	// compute modelview rotation only part
	float3x3 modelviewrot;
	modelviewrot[0]=WorldView[0].xyz;
	modelviewrot[1]=WorldView[1].xyz;
	modelviewrot[2]=WorldView[2].xyz;

	// vertex position in clip space
	OUT.hpos=mul(Po,WorldViewProj);

	// vertex position in view space (with model transformations)
	OUT.vpos=mul(Po,WorldView).xyz;

	// light position in view space
	float4 lp=float4(lightposn.xyz,1);
	OUT.lightpos=mul(lp,View);

	// tangent space vectors in view space (with model transformations)
	OUT.tangent=mul(IN.tangent,modelviewrot);
	OUT.binormal=mul(IN.binormal,modelviewrot);
	OUT.normal=mul(IN.normal,modelviewrot);
	
	// copy color and texture coordinates
	OUT.color.xyzw = 1.0 ; // IN.color;
	OUT.UV = TileCount * IN.texcoord.xy;

	return OUT;
}
VertexShader vsArray[2] = { compile vs_2_0 vertex_shader(), 
                            compile vs_2_0 vertex_shader_skin(),
};

/************ PIXEL SHADERS ******************/


float4 normal_map(
	VertexOutput IN,
	uniform sampler2D texmap,
	uniform sampler2D reliefmap) : COLOR
{
	float2 uv=IN.UV; // *TileCount;
	float4 normal=tex2D(reliefmap,uv);
	normal.xyz-=0.5; 
	
	// transform normal to world space
	normal.xyz=normalize(normal.x*IN.tangent-normal.y*IN.binormal+normal.z*IN.normal);
	
	// color map
	float4 color=tex2D(texmap,uv);

	// view and light directions
	float3 v = normalize(IN.vpos);
	float3 l = normalize(IN.lightpos.xyz-IN.vpos);

	// compute diffuse and specular terms
	float att=saturate(dot(l,IN.normal.xyz));
	float diff=saturate(dot(l,normal.xyz));
	float spec=saturate(dot(normalize(l-v),normal.xyz));

	// compute final color
	float4 finalcolor;
	finalcolor.xyz=ambient*color.xyz+
		att*(color.xyz*lightcolor.xyz*diff+lightcolor*gloss*pow(spec,PhongExp));
	finalcolor.w=1.0;

	return finalcolor;
}

float4 parallax_map(
	VertexOutput IN,
	uniform sampler2D texmap,
	uniform sampler2D reliefmap) : COLOR
{
   	// view and light directions
	float3 v = normalize(IN.vpos);
	float3 lightvector = IN.lightpos.xyz-IN.vpos;
	float  lightdistance = length(lightvector);
	float falloff = saturate(1 -lightdistance/lightradius);
	float3 l = normalize(lightvector);

	float2 uv = IN.UV; // *TileCount;

	// parallax code
	float3x3 tbn = float3x3(IN.tangent,IN.binormal,IN.normal);
	float height = tex2D(reliefmap,uv).w * 0.06 - 0.03;
	uv += height * mul(tbn,v);

	// normal map
	float4 normal=tex2D(reliefmap,uv);
	normal.xyz-=0.5; 
	
	// transform normal to world space
	normal.xyz=normalize(normal.x*IN.tangent-normal.y*IN.binormal+normal.z*IN.normal);

	// color map
	float4 color=tex2D(texmap,uv);

	// compute diffuse and specular terms
	float att= 1.0f; // previously was: saturate(dot(l,IN.normal.xyz));
	float diff=saturate(dot(l,normal.xyz));
	float spec=saturate(dot(normalize(l-v),normal.xyz));

	// compute final color
	float4 finalcolor;
	finalcolor.xyz=ambient*color.xyz+
		falloff*att*(color.xyz*lightcolor.xyz*diff+lightcolor*gloss*pow(spec,PhongExp));
	finalcolor.w=1.0;

	return finalcolor;
}


float4 pixel_shader_unlit(  // based on parallax map
	VertexOutput IN,
	uniform sampler2D texmap,
	uniform sampler2D reliefmap) : COLOR
{
   	// view and light directions
	float3 v = normalize(IN.vpos);
	//float3 lightvector = IN.lightpos.xyz-IN.vpos;
	//float  lightdistance = length(lightvector);
	//float falloff = saturate(1 -lightdistance/lightradius);
	//float3 l = normalize(lightvector);

	float2 uv = IN.UV; // *TileCount;

	// parallax code
	float3x3 tbn = float3x3(IN.tangent,IN.binormal,IN.normal);
	float height = tex2D(reliefmap,uv).w * 0.06 - 0.03;
	uv += height * mul(tbn,v);
	// color map
	float4 color=tex2D(texmap,uv);

	// compute final color
	float4 finalcolor;
	finalcolor.xyz=ambient*color.xyz;
	finalcolor.w=1.0;

	return finalcolor;
}




float4 pixel_shader_light(
	VertexOutput IN,
	uniform sampler2D texmap,
	uniform sampler2D reliefmap) : COLOR
{
   	// view and light directions
	float3 v = normalize(IN.vpos);
	float3 lightvector = IN.lightpos.xyz-IN.vpos;
	float  lightdistance = length(lightvector);
	float falloff = 1 -lightdistance*lightradiusinv;
	clip(falloff);
	//falloff=saturate(falloff);
	float3 l = normalize(lightvector);
	float  att= (8*dot(l,IN.normal.xyz));  // 1.0f; // previously was:
	clip(att);
	att=saturate(att);
	float2 uv = IN.UV; // *TileCount;

	// parallax code
	float3x3 tbn = float3x3(IN.tangent,IN.binormal,IN.normal);
	float height = tex2D(reliefmap,uv).w * 0.06 - 0.03;
	uv += height * mul(tbn,v);

	// normal map
	float4 normal=tex2D(reliefmap,uv);
	normal.xyz-=0.5; 
	
	// transform normal to world space
	normal.xyz=normalize(normal.x*IN.tangent-normal.y*IN.binormal+normal.z*IN.normal);

	// color map
	float4 color=tex2D(texmap,uv);

	// compute diffuse and specular terms
	float diff=saturate(dot(l,normal.xyz));
	float spec=saturate(dot(normalize(l-v),normal.xyz));

	// compute final color
	float4 finalcolor;
	//att=1.0f;
	finalcolor.xyz= falloff*att*(color.xyz*lightcolor.xyz*diff+lightcolor*(gloss*pow(spec,PhongExp))); // + ambient*color.xyz
	finalcolor.w=1.0;

	return finalcolor;
}

float ray_intersect_rm(			// use linear and binary search
      in sampler2D reliefmap,
      in float2 dp, 
      in float2 ds)
{
   const int linear_search_steps=15;
   
   // current size of search window
   float size=1.0/linear_search_steps;
   // current depth position
   float depth=0.0;

   // search front to back for first point inside object
   for( int i=0;i<linear_search_steps-1;i++ )
   {
		float4 t=tex2D(reliefmap,dp+ds*depth);

		if (depth<t.w)
			depth+=size;
   }
   
   const int binary_search_steps=5;

   // recurse around first point (depth) for closest match
   for( int i=0;i<binary_search_steps;i++ )
   {
		size*=0.5;
		float4 t=tex2D(reliefmap,dp+ds*depth);
		if (depth<t.w)
			depth+=2*size;
		depth-=size;
   }

   return depth;
}

float ray_intersect_rm_lin(			// only linear search for shadows
      in sampler2D reliefmap,
      in float2 dp, 
      in float2 ds)
{
   const int linear_search_steps=15;

   // current size of search window
   float size=1.0/linear_search_steps;
   // current depth position
   float depth=0.0;

   // search front to back for first point inside object
   for( int i=0;i<linear_search_steps-1;i++ )
   {
		float4 t=tex2D(reliefmap,dp+ds*depth);

		if (depth<t.w)
			depth+=size;
   }
   
   return depth;
}

float4 relief_map_shadows(
	VertexOutput IN,
	uniform sampler2D texmap,
	uniform sampler2D reliefmap) : COLOR
{
	float4 t,c;
	float3 p,v,l,s;
	float2 dp,ds,uv;
	float d,a;

	// ray intersect in view direction
	p  = IN.vpos;
	v  = normalize(p);
	a  = dot(IN.normal,-v);
	s  = float3(dot(v,IN.tangent.xyz), dot(v,IN.binormal.xyz), a);
	s  *= Depth/a;
	ds = s.xy;
	dp = IN.UV; // *TileCount;
	d  = ray_intersect_rm(reliefmap,dp,ds);

	// get rm and color texture points
	uv=dp+ds*d;
	t=tex2D(reliefmap,uv);
	c=tex2D(texmap,uv);

	// expand normal from normal map in local polygon space
	t.xyz-=0.5;
	t.xyz=normalize(t.x*IN.tangent.xyz-t.y*IN.binormal.xyz+t.z*IN.normal);

	// compute light direction
	p += v*d/(a*Depth);
	l=normalize(p-IN.lightpos.xyz);

	// compute diffuse and specular terms
	float att=saturate(dot(-l,IN.normal));
	float diff=saturate(dot(-l,t.xyz));
	float spec=saturate(dot(normalize(-l-v),t.xyz));

	// ray intersect in light direction
	dp+= ds*d;
	a  = dot(IN.normal,-l);
	s  = float3(dot(l,IN.tangent.xyz),dot(l,IN.binormal.xyz),a);
	s *= Depth/a;
	ds = s.xy;
	dp-= ds*d;
	float dl = ray_intersect_rm_lin(reliefmap,dp,s.xy);
	if (dl<d-0.05) // if pixel in shadow
	{
	  diff*=dot(ambient.xyz,float3(1.0,1.0,1.0))*0.333333;
	  spec=0;
	}

	// compute final color
	float4 finalcolor;
	finalcolor.xyz=ambient*c+att*(c*lightcolor*diff+lightcolor*gloss*pow(spec,PhongExp));
	finalcolor.w=1.0;

	return finalcolor;
}

float4 relief_map(
	VertexOutput IN,
	uniform sampler2D texmap,
	uniform sampler2D reliefmap) : COLOR
{
	float4 t,c;
	float3 p,v,l,s;
	float2 dp,ds,uv;
	float d,a;

	// ray intersect in view direction
	p  = IN.vpos;
	v  = normalize(p);
	a  = dot(IN.normal,-v);
	s  = float3(dot(v,IN.tangent.xyz), dot(v,IN.binormal.xyz), a);
	s  *= Depth/a;
	ds = s.xy;
	dp = IN.UV; // *TileCount;
	d  = ray_intersect_rm(reliefmap,dp,ds);

	// get rm and color texture points
	uv=dp+ds*d;
	t=tex2D(reliefmap,uv);
	c=tex2D(texmap,uv);

	// expand normal from normal map in local polygon space
	t.xyz-=0.5;
	t.xyz=normalize(t.x*IN.tangent.xyz-t.y*IN.binormal.xyz+t.z*IN.normal);

	// compute light direction
	p += v*d/(a*Depth);
	l=normalize(p-IN.lightpos.xyz);

	// compute diffuse and specular terms
	float att=saturate(dot(-l,IN.normal));
	float diff=saturate(dot(-l,t.xyz));
	float spec=saturate(dot(normalize(-l-v),t.xyz));

	// compute final color
	float4 finalcolor;
	finalcolor.xyz=ambient*c+att*(c*lightcolor*diff+lightcolor*gloss*pow(spec,PhongExp));
	finalcolor.w=1.0;

	return finalcolor;
}

/********************* TECHNIQUES **************/


technique parallax_mapping //  t0
{
    pass p0 
    {
		DEPTHBIAS = 0;
		TWOSIDEDSTENCILMODE = FALSE;
		ZENABLE = TRUE;
		CullMode = CW;
		STENCILENABLE=    FALSE  ;
		ALPHABLENDENABLE= FALSE ;
		SHADEMODE = GOURAUD ;
		ZWRITEENABLE =     TRUE ;
		//VertexShader = compile vs_2_0 vertex_shader();
		VertexShader = vsArray[useskin];
		PixelShader  = compile ps_2_0 parallax_map(ColorSamp,ReliefSamp);
    }
}

technique mirrored //  t0
{
    pass p0 
    {
		DEPTHBIAS = 0;
		TWOSIDEDSTENCILMODE = FALSE;
		ZENABLE = TRUE;
		CullMode = CCW;
		STENCILENABLE=    FALSE  ;
		ALPHABLENDENABLE= FALSE ;
		SHADEMODE = GOURAUD ;
		ZWRITEENABLE =     TRUE ;
		VertexShader = compile vs_2_0 vertex_shader();
		PixelShader  = compile ps_2_0 parallax_map(ColorSamp,ReliefSamp);
    }
}

technique unlit 
{
	pass p0
	{
		DEPTHBIAS = 0;
		SLOPESCALEDEPTHBIAS	= 0;	
		TWOSIDEDSTENCILMODE = FALSE;
		ZENABLE = TRUE;
		ZFUNC = LESSEQUAL ;
		ZWRITEENABLE =     TRUE ;
		CullMode = CW;
		STENCILENABLE=    FALSE  ;
		ALPHABLENDENABLE= FALSE ;
		SHADEMODE = GOURAUD ;
		PixelShader  = compile ps_2_0 pixel_shader_unlit(ColorSamp,ReliefSamp);
		VertexShader = vsArray[useskin]; //	VertexShader = compile vs_2_0 vertex_shader();
	}
}

technique lightpass
{
	pass p0
	{
		DepthBias = -0.00001f;  // nvidia 6200go and ati x800 doesn't need this , but other ati cards may need it if clipped polys aren't coplanar 
		//DEPTHBIAS = 0;
		//SLOPESCALEDEPTHBIAS	= 0;	
		STENCILENABLE=    TRUE  ;
		TWOSIDEDSTENCILMODE = FALSE;
		CCW_STENCILPASS=KEEP;
		STENCILPASS=     KEEP;
		STENCILFAIL=     KEEP;
		STENCILZFAIL=     KEEP;
		ZWRITEENABLE=     FALSE ;
		ZENABLE = TRUE;
		ZFUNC = LESSEQUAL ;
		STENCILREF=  0x01 ;
		STENCILFUNC= GREATER; // since unlike shadowpoly we want to draw where shadow isn't  // LESSEQUAL ;
		FOGENABLE=        FALSE ;
		ALPHABLENDENABLE= TRUE  ;
		SHADEMODE = GOURAUD ;
		SRCBLEND=  ONE ;
		DESTBLEND= ONE ;
		CullMode = CW;
		FILLMODE=SOLID;
		PixelShader  = compile ps_2_0 pixel_shader_light(ColorSamp,ReliefSamp);
		VertexShader = vsArray[useskin];
	}
}

technique  normal_mapping
{
    pass p0 
    {
    	CullMode = CW;
		VertexShader = vsArray[useskin];
		PixelShader  = compile ps_2_0 normal_map(ColorSamp,ReliefSamp);
    }
}


technique relief_mapping 
{
    pass p0 
    {
    	CullMode = CW;
		VertexShader = vsArray[useskin];
		PixelShader  = compile ps_3_0 relief_map(ColorSamp,ReliefSamp);
    }
}

technique relief_mapping_shadows
{
    pass p0 
    {
    	CullMode = CW;
		VertexShader = vsArray[useskin];
		PixelShader  = compile ps_3_0 relief_map_shadows(ColorSamp,ReliefSamp);
    }
}


technique test // test 
{
    pass p0 
    {
    	CullMode = CW;
		VertexShader = vsArray[useskin];
		PixelShader  = compile ps_2_0 pixel_shader_test();
    }
}

//-------- deferred lighting --------------


struct VtoP_deferred  // all elements are in world space
{
	float4 position : POSITION;  
	float2 tex      : TEXCOORD;
	//float4 color  : COLOR;
	float4 worldpos : TexCoord1;
	float3 tangent  : TEXCOORD2;
	float3 binormal : TEXCOORD3;
	float3 normal   : TEXCOORD4;
	float4 position2: TEXCOORD5;  // identical to position, but accessible from pixel shader
};

struct psOut_deferred
{
	float4 diffuse  : Color0;
	float4 normals  : Color1;  // packed into range 0..1 on each xyz
	float4 position : Color2;  // actually we just use one F32 channel for depth info, but its used to derive position later on
};
VtoP_deferred vertex_shader_deferred_setup(const VS_INPUT IN)
{
	VtoP_deferred OUT;
	OUT.position = mul(IN.position, WorldViewProj);
	OUT.position2= mul(IN.position, WorldView);
	OUT.worldpos = mul(IN.position, World);
	OUT.tangent  = mul(float4(IN.tangent.xyz ,0.0f),World).xyz;
	OUT.binormal = mul(float4(IN.binormal.xyz,0.0f),World).xyz;
	OUT.normal   = mul(float4(IN.normal.xyz  ,0.0f),World).xyz;
	OUT.tex      = IN.texcoord;
	return OUT;
}
psOut_deferred pixel_shader_deferred_setup(VtoP_deferred In,uniform sampler2D texmap,uniform sampler2D reliefmap)
{
	psOut_deferred Out;
	
	
	float3 viewdir  = normalize(ViewInv[3].xyz - In.worldpos.xyz);

	float2 uv = In.tex; 

	// parallax code
	float3x3 tbn = float3x3(In.tangent,In.binormal,In.normal);
	float height = tex2D(reliefmap,uv).w * 0.06 - 0.03; 
	uv += height * mul(tbn,-viewdir);  
	
	float4 d = tex2D(texmap, uv);
	Out.diffuse = d;
	// detemine normal
	float3 normal=tex2D(reliefmap,uv).xyz;
	normal.xyz-=0.5; 
	normal.xyz=normalize(normal.x*In.tangent-normal.y*In.binormal+normal.z*In.normal); // transform normal to world space
	// normal.xyz *= dot(normalize(In.normal).xyz,normal.xyz); // was an unsuccessful try to approx fabio's attenuation att parameter
	Out.normals.xyz = float3(0.5,0.5,0.5)+normal*0.5;
	//Out.normals.xyz =  (float3(1,1,1)+mul(2.0f * tex2D(reliefmap, uv).xyz - 1.0f, float3x3(In.tangent, In.binormal, In.normal)))*0.5f;
	Out.normals.w = 1.0f;
  	Out.position.xyzw = -length(In.worldpos-ViewInv[3].xyz); // ok this is really distance     // In.position2.z / In.position2.w;    // depth
	return Out;
}


technique deferred_setup
{
	pass p0
	{
		DEPTHBIAS = 0;
		TWOSIDEDSTENCILMODE = FALSE;
		CullMode = CW;
		STENCILENABLE=    FALSE  ;
		ALPHABLENDENABLE= FALSE ;
		SHADEMODE = GOURAUD ;
		ZWRITEENABLE =     TRUE ;
		ZENABLE = TRUE;
		ZFUNC = LESSEQUAL ;
		PixelShader  = compile ps_2_0 pixel_shader_deferred_setup(ColorSamp,ReliefSamp);
		VertexShader = compile vs_2_0 vertex_shader_deferred_setup();
	}
}




//-------------------------------------------

struct VtoPalpha
{
	float4 position : POSITION;
	float2 tex      : TEXCOORD;
	float4 color    : COLOR;  // encoded into range [0,1]
};

VtoPalpha vertex_shader_alpha(const VS_INPUT IN)
{
	VtoPalpha OUT = (VtoPalpha)0;
	float4 p = mul(IN.position, WorldViewProj);
	OUT.position = p;
	OUT.color.xyz =  1.0;
	OUT.color.w = alpha;
	OUT.tex = IN.texcoord;
	return OUT;
}

float4 pixel_shader_alpha(const VtoPalpha IN):COLOR
{
	float4 color = IN.color * tex2D(ColorSamp, IN.tex);
	color.w = alpha;
	return color;
}

technique alpha
{
	pass p0
	{
		DEPTHBIAS = 0;
		TWOSIDEDSTENCILMODE = FALSE;
		ZENABLE = TRUE;
		CullMode = CW;
		STENCILENABLE=    FALSE  ;
		ALPHABLENDENABLE= TRUE ;
		SHADEMODE = GOURAUD ;
		ZWRITEENABLE =     FALSE ;
        SRCBLEND  = SRCALPHA ;
        DESTBLEND = INVSRCALPHA ;
		PixelShader  = compile ps_2_0 pixel_shader_alpha();
		VertexShader = compile vs_2_0 vertex_shader_alpha();
	}
	pass p1
	{
		DEPTHBIAS = 0;
		TWOSIDEDSTENCILMODE = FALSE;
		ZENABLE = TRUE;
		CullMode = CCW;
		STENCILENABLE=    FALSE  ;
		ALPHABLENDENABLE= TRUE ;
		SHADEMODE = GOURAUD ;
		ZWRITEENABLE =     FALSE ;
        SRCBLEND  = SRCALPHA ;
        DESTBLEND = INVSRCALPHA ;
		PixelShader  = compile ps_2_0 pixel_shader_alpha();
		VertexShader = compile vs_2_0 vertex_shader_alpha();
	}
}

/****************************************** EOF ***/
